package com.muyu.active.domain;

import com.muyu.active.domain.model.ActivityTeamProductSkuAddModel;
import com.muyu.active.domain.model.ActivityTeamProductSkuReqModel;
import com.muyu.active.domain.model.ActivityTeamProductSkuUpdModel;
import com.muyu.active.domain.model.TeamSkuInfoModel;
import com.muyu.active.domain.req.TeamProjectSkuInfoAddReq;
import com.muyu.common.core.web.domain.BaseEntity;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.math.BigDecimal;
import java.util.function.Function;

/**
 * 商品sku主表
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@SuperBuilder
@EqualsAndHashCode(callSuper = true)
public class ActivityTeamProductSkuInfo extends BaseEntity {
  private long id;
  private long teamId;
  private Long productId;
  private String productSku;
  private Long teamStock;
  private Long remainStock;
  private BigDecimal teamPrice;

  /**
   * 构造函数的方法
   * 商品sku模型
   */
  public static ActivityTeamProductSkuInfo TeamSkuInfoBuild(TeamSkuInfoModel teamSkuInfoModel, Function<ActivityTeamProductSkuInfoBuilder,ActivityTeamProductSkuInfo> function){
    return function.apply(ActivityTeamProductSkuInfo.builder()
            .teamPrice(teamSkuInfoModel.getTeamPrice())
            .teamStock(teamSkuInfoModel.getTeamStock())
            .productSku(teamSkuInfoModel.getProductSku())
            .productId(teamSkuInfoModel.getProductId()));
  }
  /**
   * 通过模型钢构件对象
   * @param activityTeamProductSkuAddModel 模型
   * @return 对象
   */
  public static ActivityTeamProductSkuInfo modelBuild(ActivityTeamProductSkuAddModel activityTeamProductSkuAddModel) {
    return ActivityTeamProductSkuInfo.builder()
            .productId(activityTeamProductSkuAddModel.getProductId())
            .teamId(activityTeamProductSkuAddModel.getTeamId())
            .teamStock(activityTeamProductSkuAddModel.getTeamStock())
            .productSku(activityTeamProductSkuAddModel.getSku())
            .remainStock(activityTeamProductSkuAddModel.getTeamStock())
            .teamPrice(activityTeamProductSkuAddModel.getTeamPrice())
            .build();
  }
  public static ActivityTeamProductSkuInfo updateModelBuild(ActivityTeamProductSkuReqModel activityTeamProductSkuReqModel) {
    return ActivityTeamProductSkuInfo.builder()
            .id(activityTeamProductSkuReqModel.getId())
            .productId(activityTeamProductSkuReqModel.getProductId())
            .teamStock(activityTeamProductSkuReqModel.getTeamStock())
            .productSku(activityTeamProductSkuReqModel.getProductSku())
            .teamPrice(activityTeamProductSkuReqModel.getTeamPrice())
            .build();
  }

  public static ActivityTeamProductSkuInfo updModelBuild(ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel) {
    return ActivityTeamProductSkuInfo.builder()
            .id(activityTeamProductSkuUpdModel.getId())
            .teamPrice(activityTeamProductSkuUpdModel.getTeamPrice())
            .teamStock(activityTeamProductSkuUpdModel.getTeamStock())
            .remainStock(activityTeamProductSkuUpdModel.getTeamStock())
            .build();
  }

  public static ActivityTeamProductSkuInfo buildAddSku(TeamProjectSkuInfoAddReq teamProjectSkuInfoAddReq) {
    return ActivityTeamProductSkuInfo.builder()
            .teamId(teamProjectSkuInfoAddReq.getTeamId())
            .productId(teamProjectSkuInfoAddReq.getProductId())
            .productSku(teamProjectSkuInfoAddReq.getSku())
            .teamPrice(teamProjectSkuInfoAddReq.getTeamPrice())
            .teamStock(teamProjectSkuInfoAddReq.getTeamStock())
            .build();
  }

}
