package com.muyu.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.muyu.active.domain.ActivityTeamProductSkuInfo;
import com.muyu.active.domain.model.*;

import java.util.List;


/**
 * 商品拼团信息Service接口
 */
public interface ActivityTeamProductSkuInfoService extends IService<ActivityTeamProductSkuInfo> {
    public default List<ActivityTeamProductSkuInfo> acivitySkuList (Long id){
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,id);
        return this.list(lambdaQueryWrapper);
    }
    public ActivityTeamDiscountModel discountPrice(Long id);
    public TeamProductStockModel getStock(Long id);
    /**
     * 通过团购活动ID获取团购中最优惠的价格
     * @param teamId 团购ID
     * @return 优惠价格
     */
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId);
    /**
     * 通过活动ID获取 剩余库存
     *
     * @param teamId 活动ID
     * @return 库存
     */
    public TeamStockModel getStocks(Long teamId);


    /**
     * 添加团购商品SKU
     * @param activityTeamProductSkuAddModelList 团购商品SKU添加模型集合
     */
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList);

    /**
     * 通过拼团活动ID获取拼团下商品SKU集合
     * @param teamId 活动ID
     * @return 拼团商品SKU集合
     */
    public List<ActivityTeamProductSkuModel> findListByTeamId(Long teamId);

    /**
     * 修改团购商品sku信息
     *
     * @param activityTeamProductSkuUpdModel
     * @return
     */
    public boolean update(ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel);

    /**
     * 修改团购商品sku信息（批量修改）
     * @param activityTeamProductSkuUpdModelList
     * @return
     */
    public void batchUpdate(List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuUpdModelList);
}
