package com.muyu.service.impl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.active.domain.ActivityTeamProductSkuInfo;
import com.muyu.active.domain.model.*;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import com.muyu.service.ActivityTeamProductSkuInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

/**
 * 商品拼团信息Service业务层处理
 */
@Slf4j
@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo>  implements ActivityTeamProductSkuInfoService {
    @Autowired
    private ProjectSkuCache projectSkuCache;
    /**
     * 通过团购活动ID获取团购中最优惠的价格
     * @param teamId 团购ID
     * @return 优惠价格
     */
    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        //  List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.acivitySkuList(teamId);
        // 优惠模型集合
        Optional<TeamProductDiscountPriceModel> discountPriceModelOptional = teamProductSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
                    ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                    return TeamProductDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
                }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());

        if (discountPriceModelOptional.isEmpty()){
            throw new ServiceException("团购活动下没有商品绑定");
        }
        return discountPriceModelOptional.get();
    }


    @Override
    public ActivityTeamDiscountModel discountPrice(Long id) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,id);
        List<ActivityTeamProductSkuInfo> list = this.list(lambdaQueryWrapper);
        if (list.size()>0){
            Optional<ActivityTeamDiscountModel> min = list.stream().map(activityTeamProductSkuInfo -> {
                ProjectSkuInfo projectSkuInfo = projectSkuCache.getData(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                return ActivityTeamDiscountModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
            }).min((o1, o2) -> Double.valueOf(100 * o1.getDiscount() - 100 * o2.getDiscount()).intValue());
            if (min.isEmpty()){
                throw new ServiceException("没有优惠");
            }
            return min.get();
        }
        return ActivityTeamDiscountModel.builder()
                .teamPrice(BigDecimal.valueOf(0))
                .productPrice(BigDecimal.valueOf(0))
                .discount(0.00)
                .build();

    }

    @Override
    public TeamProductStockModel getStock(Long id) {
        // List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.acivitySkuList(id);
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L, Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L, Long::sum))
                .build();
    }



    @Override
    public TeamStockModel getStocks(Long id) {
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = this.acivitySkuList(id);
        return TeamStockModel.builder()
                .teamStock(activityTeamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L,Long::sum))
                .remainStock(activityTeamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L,Long::sum))
                .build();
    }

    @Override
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList) {
        if (activityTeamProductSkuAddModelList == null || activityTeamProductSkuAddModelList.isEmpty()) {
            throw new ServiceException("入参添加商品规格模型，不可为空");
        }
        this.saveBatch(
                activityTeamProductSkuAddModelList.stream().map(ActivityTeamProductSkuInfo::modelBuild).toList()
        );
    }

    @Override
    public List<ActivityTeamProductSkuModel> findListByTeamId(Long teamId) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,teamId);
        List<ActivityTeamProductSkuInfo> list = this.list(queryWrapper);
        return list.stream().map(ActivityTeamProductSkuModel::FindBuild).toList();
//        return list.stream().map(activityTeamProductSkuInfo -> ActivityTeamProductSkuModel.FindBuild(activityTeamProductSkuInfo)).toList();
    }

    /**
     * 单表修改
     * @param activityTeamProductSkuUpdModel
     * @return
     */
    @Override
    public boolean update(ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel) {
        LambdaUpdateWrapper<ActivityTeamProductSkuInfo> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamPrice,activityTeamProductSkuUpdModel.getTeamPrice());
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamStock,activityTeamProductSkuUpdModel.getTeamStock());
        updateWrapper.eq(ActivityTeamProductSkuInfo::getId,activityTeamProductSkuUpdModel.getId());
        return this.update(updateWrapper);
    }

    @Override
    public void batchUpdate(List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuUpdModelList) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfos = activityTeamProductSkuUpdModelList.stream()
                .map(ActivityTeamProductSkuInfo::updModelBuild)
                .toList();
        this.updateBatchById(teamProductSkuInfos);
  }
}
